@@ -29,15 +29,20 @@ read.aggregation <- function(filename, mnemonics = NULL, ...,
 
   ## Order components by the order observed in the file
   data$component <- factor(data$component, levels=unique(data$component))
+
+  ## Read metadata and mnemonic-ize it as well
+  metadata <- read.metadata(filename, comment.char = comment.char)
+  names(metadata) <- map.mnemonics(names(metadata), mnemonics)$labels
   
-  data
+  list(data = data, metadata = metadata)
 }
 
 
 ## Generates pretty scales for the data.
 ## layout is a 2-element vector: c(num_rows, num_cols) for the xyplot
 ## num_panels is the number of panels/packets in the trellis
-panel.scales <- function(data, layout, num_panels, x.axis = FALSE) {
+panel.scales <- function(data, layout, num_panels, x.axis = FALSE,
+                         significance = FALSE) {
   components <- levels(data$component)
   num_components <- length(components)
 
@@ -51,6 +56,8 @@ panel.scales <- function(data, layout, num_panels, x.axis = FALSE) {
     component_subset <- subset(data, component == cur_component)
     min.x <- min(component_subset$offset, na.rm=TRUE)
     max.x <- max(component_subset$offset, na.rm=TRUE)
+    if (!is.finite(min.x)) 
+      print(component_subset)
     at.x.pretty <- at.pretty(from=min.x, to=max.x, length=5, largest=TRUE)
     at.x <- c(at.x, at.x.pretty)
     ## Cludgy shrink of axes to compensate for automatic trellis expansion
@@ -73,8 +80,18 @@ panel.scales <- function(data, layout, num_panels, x.axis = FALSE) {
     }
 
   range.y <- range(data$overlap, na.rm=TRUE)
-  min.y <- min(range.y[1], 0)
+
+  ## Extend range for rug
+  ngroups <- nlevels(data$group)
+  rug.height <-
+    if (significance && ngroups > 1)
+      diff(range.y) / (1 - get.rug.start(ngroups + 2)) - diff(range.y)
+    else
+      0
+  
+  min.y <- min(range.y[1], 0) - rug.height
   max.y <- max(range.y[2], 0)
+  
   limits.y <- extendrange(c(min.y, max.y))
   at.y <- unique(round(c(min.y, 0, max.y), digits = 2))
   scales <- list(x = list(relation = "free",
@@ -85,7 +102,8 @@ panel.scales <- function(data, layout, num_panels, x.axis = FALSE) {
                  y = list(alternating = c(2, 0),
                           tck = c(0, 1),
                           limits = limits.y,
-                          at = at.y))
+                          at = at.y)
+                 )
 
   scales
 }
@@ -122,7 +140,7 @@ cast.aggregation <- function(data.df) {
   cast(data.df, group + component + offset ~ label, value = "count")
 }
 
-calc.signif <- function(count, total, random.prob) {
+calc.pvals <- function(count, total, random.prob) {
   ## Calculate "significance" of count and total given
   ## current random.prob.
   if (!is.finite(count) || !is.finite(total)) {
@@ -152,11 +170,7 @@ calc.signif <- function(count, total, random.prob) {
       }
     }
   signif <- min(signif, 1)
-##   signif <- -log10(signif)
-##   signif[!is.finite(signif)] <- Inf
-##   signif[signif > 20] <- 20
-##   signif <- if(count < expected) -signif else signif
-  
+
   signif
 }
 
@@ -164,69 +178,126 @@ calc.signif <- function(count, total, random.prob) {
 ## normalizing the counts over all labels and by the sizes of the labels
 ## as well as calculating a significance level for each row.
 process.counts <- function(data, label.sizes, pseudocount = 1,
-                           normalize = TRUE, pval.thresh = 0.001) {
+                           normalize = TRUE) {
   if (!is.vector(label.sizes)) stop("Expected vector of label sizes")
   total.size <- sum(label.sizes)
   if (length(total.size) != 1) stop("Error summing label size vector")
   if (!is.finite(total.size)) stop("Unexpected sum of sizes (not finite)")
 
-  ##data.mat <- cast.aggregation(data)
-  
   ## Ideally, a multinomial test would be applied once for each bin
   ## (testing whether the distribution of overlaps by label is as expected),
   ## but the label-wise binomial seems a decent approximation.
-  labels.sum <- with(data, aggregate(count, list(offset, component),
-                               sum))$x
+  
+  ## Get sum over labels for each component and offset
+  labels.sum <- with(data, aggregate(count, list(offset = offset,
+                                                 component = component,
+                                                 group = group), sum))
+  names(labels.sum) <- gsub("^x$", "sum", names(labels.sum))
+  
   for (label in levels(data$label)) {
     random.prob <- label.sizes[label] / total.size
+##    if (label == "NONE") cat(paste("\n\nNONE:\nrandom.prob:", random.prob))
     cur.rows <- data$label == label
-    cur.counts <- data$count[cur.rows]
-
-    calc.row.signif <- function(row) {
+    cur.data <- data[cur.rows,]
+    ## Add order row to keep track of order extracted
+    ## (so we can reorder for re-insertion)
+    cur.data$order <- 1:nrow(cur.data)
+    ## Add sum over labels as data frame column
+    cur.data <- merge(cur.data, labels.sum)
+    ## Reorder merged data back to the order extracted
+    cur.data <- cur.data[order(cur.data$order),]
+    calc.row.pvals <- function(row) {
       count <- row[1]
       total <- row[2]
-      calc.signif(count, total, random.prob)
+      calc.pvals(count, total, random.prob)
     }
 
+##    if (label == "NONE") print(cur.data[1:1,])
+    data$pval[cur.rows] <- apply(subset(cur.data, select = c(count, sum)), 1,
+                                 calc.row.pvals)
+    
     if (normalize) {
-      data$count[cur.rows] <- log2((cur.counts / labels.sum + 1) /
-                                   (random.prob + 1))
+      cur.enrichments <- log2((cur.data$count / cur.data$sum + 1) /
+                              (random.prob + 1))
+      ## Any NaN regions (from 0/0) are masked
+      cur.enrichments[is.nan(cur.enrichments)] <- 0
+      if (any(cur.enrichments < -1) || any(cur.enrichments > 1)) {
+        cat(paste("Enrichment value out of range: [-1, 1]",
+                  paste("label:", label),
+                  sep = "\n"))
+        stop()
+      }
+##       if (label == "NONE") {
+##         cat("\ncur.enrichments:")
+##         cat(cur.enrichments[1:3])
+##       }
+      
+      data$count[cur.rows] = cur.enrichments
     }
-    pvals <- apply(cbind(cur.counts, labels.sum), 1, calc.row.signif)
-    data$significant[cur.rows] <- (is.finite(pvals) & (pvals < pval.thresh))
-    ##data$count[cur.rows] <- -log10(pvals)
   }
-  
   data
 }
 
+panel.significance <- function(x, y, height, signif, col = plot.line$col,
+                               lty = plot.line$lty, lwd = plot.line$lwd, ...) {
+  ## signif should be a boolean vector as long as x
+  n <- length(x)
+  if (n != length(signif)) stop()
+  require("grid", quietly = TRUE)
+  x.units <- "native"
+  y.units <- "npc"
+  plot.line <- trellis.par.get("plot.line")
+  
+  ## Identify contiguous blocks of significance
+  starts <- x[c(signif[1], !signif[-n] & signif[-1])]
+  ends <- x[c(signif[-n] & !signif[-1], signif[n])]
 
-panel.aggregation <- function(x, y, significant, ngroups, groups = NULL,
+  gp <- gpar(col = col, lty = 1, lwd = 0, fill = col)
+  x0 <- unit(starts - 0.5, x.units)
+  #x1 <- unit(ends + 0.5, x.units)
+  width <- unit(ends + 0.5 - starts, x.units)
+  y0 <- unit(y, y.units)
+  height <- unit(height, y.units)
+#  grid.segments(x0, y0, x1, y0, gp = gp)
+  grid.rect(x0, y0, width, height, just = c("left", "bottom"), gp = gp)
+}
+
+get.rug.start <- function(group.number, rug.height = 0.03, rug.spacing = 0.015,
+                          rug.offset = 0.025, ...) {
+  rug.offset + (rug.spacing + rug.height) * (group.number - 1)
+}
+
+panel.aggregation <- function(x, y, ngroups, signif = NULL, groups = NULL,
                               subscripts = NULL, font = NULL, col = NULL,
                               col.line = NULL, pch = NULL,
-                              group.number = NULL, ...) {
+                              group.number = NULL, rug.height = 0.03, ...) {
   ## hide 'font' from panel.segments, since it doesn't like having
   ## font and fontface
   panel.refline(h = 0)
 
-  significant <- as.logical(significant)[subscripts]
-  significant[!is.finite(significant)] <- FALSE
-  
   x <- as.numeric(x)
   y <- as.numeric(y)
-  if (any(significant)) {
-    ## Only shade region for first.
-    fill.col <- "black"
-    #fill.col <- if (ngroups == 1) "black" else col.line
-    if (ngroups == 1) {
-      y.sig <- y
-      y.sig[!significant] <- 0
-      panel.polygon(cbind(c(min(x), x, max(x)), c(0, y.sig, 0)),
-                    col = fill.col)
-    } else {
-      x.sig <- x[significant]
-      y.sig <- y[significant]
-      panel.points(x.sig, y.sig, col = fill.col, pch = "*")
+
+  if (!is.null(signif)) {
+    signif <- as.logical(signif)[subscripts]
+    signif[!is.finite(signif)] <- FALSE
+  
+    if (any(signif)) {
+      ## Only shade region for first.
+      if (ngroups == 1) {
+        fill.col <- "black"
+        y.sig <- y
+        y.sig[!signif] <- 0
+        panel.polygon(cbind(c(min(x), x, max(x)), c(0, y.sig, 0)),
+                      col = fill.col)
+      } else {
+        ##x.sig <- x[signif]
+        ##y.sig <- y[signif]
+        ##panel.points(x.sig, y.sig, col = fill.col, pch = "*")
+        rug.start <- get.rug.start(group.number, rug.height = rug.height, ...)
+        panel.significance(x, rug.start, rug.height, signif = signif,
+                           col = col.line, ...)
+      }
     }
   }
 ##  panel.xyplot(x, y, groups = groups, subscripts = subscripts, ...)
@@ -234,7 +305,8 @@ panel.aggregation <- function(x, y, significant, ngroups, groups = NULL,
                pch = pch, ...)
 }
 
-get.metadata.label.sizes <- function(metadata, data) {
+get.label.sizes <- function(data, metadata) {
+  if (length(metadata) == 0) stop("Missing aggregation metadata")
   label.sizes <- NULL
   for (label in levels(data$label)) {
     label.size.raw <- metadata[[as.character(label)]]
@@ -248,18 +320,26 @@ get.metadata.label.sizes <- function(metadata, data) {
   label.sizes
 }
 
+calculate.signif <- function(data, fdr.level = 0.05) {
+  require("qvalue", quietly = TRUE)
+  qobj <- qvalue(p = data$pval, pi0.method = "bootstrap", fdr.level = fdr.level)
+  qobj$significant
+}
+
 ## Plots overlap vs position for each label
-##   data: a data frame with fields: overlap, offset, label
+##   data: a data frame with fields: count, offset, label
 ##   spacers should be a vector of indices, where a spacer will be placed after
 ##     that component (e.g. c(1, 3) will place spacers after the first and third
 ##     components
-xyplot.aggregation <- function(data,
-    metadata = NULL,
+xyplot.aggregation <- function(agg.data = NULL,
+    data = agg.data$data,
     x = overlap ~ offset | component * label,
-    spacers = metadata[["spacers"]],
+    metadata = agg.data$metadata,
+    spacers = metadata$spacers,
     normalize = TRUE,
+    significance = FALSE,
     x.axis = FALSE,  # Show x axis
-    pval.thresh = 0.001,
+    fdr.level = 0.05,
     text.cex = 1,
     spacing.x = 0.4,
     spacing.y = 0.4,
@@ -273,24 +353,30 @@ xyplot.aggregation <- function(data,
     strip = strip.custom(horizontal = FALSE),
     strip.height = 10,
     xlab = NULL,
-    ylab = if (normalize) "Enrichment {log2[(fObs + 1)/(fRand + 1)]}"
+    ylab = if (normalize)
+              expression(paste("Enrichment: ",
+                  log[2], group("[", (f[plain(obs)] + 1)/(f[plain(rand)] + 1),
+                                 "]")))
            else "Count",
-    sub = if (normalize) paste("Black regions are significant with p<",
-                               pval.thresh, sep = "")
-          else NULL,
+    sub = if (significance) {
+            substitute(
+              expression(paste(FORMAT, " regions are significant with ",
+                               italic(q) <= FDR)),
+              list(FORMAT = if (ngroups > 1) "Rug" else "Black",
+                   FDR = fdr.level))
+          } else {
+            NULL
+          },
     ...)
 {
-  metadata <- as.list(metadata)
+  label.sizes <- get.label.sizes(data, metadata)
   ## Normalize and/or calculate significance if metadata exists
-  if (length(metadata) > 0) {
-    label.sizes <- get.metadata.label.sizes(metadata, data)
-    data <- process.counts(data, label.sizes, pval.thresh = pval.thresh,
-                             normalize = normalize)
-  } else {
-    stop("Cannot normalize/calculate significance without metadata")
+  data <- process.counts(data, label.sizes, normalize = normalize)
+  if (significance) {
+    data$signif <- calculate.signif(data, fdr.level)
   }
-
-  colnames(data) <- gsub("^count$", "overlap", colnames(data))
+  
+  names(data) <- gsub("^count$", "overlap", names(data))
   data$overlap[!is.finite(data$overlap)] <- 0
 
   ## Determine panel layout
@@ -313,7 +399,8 @@ xyplot.aggregation <- function(data,
 
   ## Separate distinct groups
   spaces.x <- rep(0, num_components - 1)
-  if (is.numeric(spacers) && length(spacers) > 0) {
+  if (!is.null(spacers) && length(spacers) > 0) {
+    spacers <- as.integer(spacers)
     if (any(spacers < 1 | spacers >= num_components)) {
       stop("Spacer vector should only contain values in the range [1, ",
            num_components - 1,
@@ -324,7 +411,8 @@ xyplot.aggregation <- function(data,
   between <- list(x = c(spaces.x, spacing.x), 
                   y = spacing.y)
 
-  scales <- panel.scales(data, layout, num_panels, x.axis = x.axis)
+  scales <- panel.scales(data, layout, num_panels, x.axis = x.axis,
+                         significance = significance)
   axis.panel <- rep(c(0, 1), c(num_cols - 1, 1))
 
   ## Make top strips longer
@@ -333,7 +421,7 @@ xyplot.aggregation <- function(data,
   args <- list(x, data = data, type = "l", groups = quote(group),
                auto.key = auto.key, as.table = TRUE, strip = strip,
                xlab = xlab, ylab = ylab, sub = sub,
-               significant = data$significant, ngroups = ngroups,
+               signif = data$signif, ngroups = ngroups,
                panel = "panel.superpose", panel.groups = panel, ...)
 
   trellis.raw <- do.call(xyplot, args)
@@ -346,12 +434,10 @@ xyplot.aggregation <- function(data,
 
 plot.aggregation <- function(filename, mnemonics = NULL, ...,
                              comment.char = "#") {
-  data <- read.aggregation(filename, mnemonics = mnemonics)
-  metadata <- read.metadata(filename, comment.char = comment.char)
-  # Rename metadata keys with mnemonics
-  names(metadata) <- map.mnemonics(names(metadata), mnemonics)$labels
+  data <- read.aggregation(filename, mnemonics = mnemonics,
+                           comment.char = comment.char)
 
-  xyplot.aggregation(data = data, metadata = metadata, ...)
+  xyplot.aggregation(data, ...)
 }
 
 save.aggregation <- function(dirpath, namebase, tabfilename,
@@ -361,15 +447,48 @@ save.aggregation <- function(dirpath, namebase, tabfilename,
                              comment.char = "#",
                              ...) {
   mnemonics <- read.mnemonics(mnemonic_file)
-  data <- read.aggregation(tabfilename, mnemonics = mnemonics)
-  metadata <- read.metadata(tabfilename, comment.char = comment.char)
-  # Rename metadata keys with mnemonics
-  names(metadata) <- map.mnemonics(names(metadata), mnemonics)$labels
+  data <- read.aggregation(tabfilename, mnemonics = mnemonics,
+                           comment.char = comment.char)
   
-  image.size <- panel.size * ceiling(sqrt(nlevels(data$label) / 2))
+  image.size <- panel.size * ceiling(sqrt(nlevels(data$data$label) / 2))
   save.images(dirpath, namebase,
-              xyplot.aggregation(data = data, metadata = metadata, ...),
+              xyplot.aggregation(data, ...),
               width = image.size,
               height = image.size,
               clobber = clobber)
 }
+
+save.gene.aggregations <- function(dirpath, namebase1, namebase2, tabfilename,
+                                   mnemonic_file = NULL,
+                                   clobber = FALSE,
+                                   panel.size = 200,  # px
+                                   comment.char = "#",
+                                   ...) {
+  mnemonics <- read.mnemonics(mnemonic_file)
+  data <- read.aggregation(tabfilename, mnemonics = mnemonics,
+                           comment.char = comment.char)
+  
+  image.size <- panel.size * ceiling(sqrt(nlevels(data$data$label) / 2))
+  ngroup1 <- as.integer(data$metadata[["spacers"]])
+  if (ngroup1 <= 0 || ngroup1 >= nlevels(data$data$component))
+    stop("Invalid metadata for gene aggregation plots")
+
+  plot.one <- function(namebase, components.sub, ...) {
+    data.sub <- subset(data$data, component %in% components.sub, drop = TRUE)
+    data.sub$component <- factor(data.sub$component, levels=components.sub)
+    save.images(dirpath, namebase,
+                xyplot.aggregation(data = data.sub, metadata = data$metadata,
+                                   spacers = NULL, ...),
+                width = image.size,
+                height = image.size,
+                clobber = clobber)
+  }
+
+  components <- levels(data$data$component)
+  ## Transcriptional components
+  plot.one(namebase1, head(components, ngroup1), ...)
+  ## Translational components (plus flankings again)
+  plot.one(namebase2, c(components[1],
+                        tail(components, -ngroup1),
+                        components[ngroup1]), ...)
+}
