@@ -51,14 +51,17 @@ labels.log <- function(...) {
 }
 
 read.mnemonics <- function(filename, stringsAsFactors = NULL,
-                           colClasses = NULL, na.strings = NULL, ...) {
+                           colClasses = NULL, na.strings = NULL,
+                           quote = "", comment.char = "",
+                           ...) {
   filename <- as.character(filename)
   if (length(filename) > 0 && nchar(filename) > 0) {
     if (!file.exists(filename)) {
       stop(paste("Error: could not find mnemonic file:", filename))
     }
     read.delim(filename, stringsAsFactors = FALSE, colClasses = "character",
-               na.strings = NULL, ...)[, 1:2]
+               na.strings = NULL, quote = quote, comment.char = comment.char,
+               ...)[, 1:2]
   } else {
     NULL
   }
@@ -168,7 +171,7 @@ labels.classify <- function(labels) {
 relevel.mnemonics <- function(labels, mnemonics = NULL) {
   if (!is.factor(labels)) stop("Expected labels as factor")
   label.map <- map.mnemonics(levels(labels), mnemonics = mnemonics)
-  if (nunique(label.map$labels) != length(label.map$labels)) {
+  if (length(unique(label.map$labels)) != length(label.map$labels)) {
     stop(paste("Error: mnemonic replacement labels are not unique:",
                paste(label.map$labels, collapse = ", ")))
   }
@@ -281,7 +284,8 @@ theme.slide <- function() {
  res <- theme.dark2()
 
  additions <-
-   list(axis.text = list(cex = 1.3),
+   list(axis.text.x = list(cex = 1.3),
+        axis.text.y = list(cex = 1.3),
         par.main.text = list(cex = 2),
         par.sub.text = list(cex = 2),
         layout.heights = list(axis.top = 1.5, axis.bottom = 1.25),
@@ -334,26 +338,34 @@ calc.slide.scale <- function(width, height, pivot.dim = 1000) {
   125 * scaler
 }
 
-save.image <- 
-  function(basename, ext, dirname, device, as.slide = FALSE, ...,
-           clobber = FALSE) 
-{
-  filename.ext <- extpaste(basename, ext)
-  filename.fq <- file.path(dirname, filename.ext)
-
-  if (!clobber && file.exists(filename.fq)) {
-    stop(paste("Error:", filename.fq, "already exists!",
-               "Specify --clobber to overwrite."))
-  }
-  
+print.image <- function(filepath, device, as.slide = FALSE, ...) {
   if (as.slide) {
-    device(filename.fq, ...)
+    device(filepath, ...)
     print(as.slide())
     dev.off()
   } else {
-    dev.print(device=device, file=filename.fq, ...)
+    dev.print(device=device, file=filepath, ...)
   }
 
+  filepath
+}
+
+save.image <- function(basename, ext, dirname, device, ..., clobber = FALSE) {
+  filename.ext <- extpaste(basename, ext)
+  filepath <- file.path(dirname, filename.ext)
+
+  if (!clobber && file.exists(filepath)) {
+    cat(paste("Error:", filepath, "already exists.",
+              "Image will not be overwritten.",
+              "Specify --clobber to overwrite."))
+  } else {
+    tryCatch(print.image(filepath, device, ...),
+             error = function(e) {
+               cat(paste("Error creating image: ", filepath,
+                         ". Error message: ", e$message, sep = ""))
+             })
+  }
+  
   filename.ext
 }
 
@@ -393,7 +405,7 @@ dev.print.images <- function(basename, dirname,
     filename.slide <-
       save.image(basename, "slide.png", dirname, device.png, 
                  width = width.slide, height = height.slide, 
-                 units = "px", as.slide = pdf.as.slide,
+                 units = "px", as.slide = TRUE,
                  ..., clobber = clobber)
   }
 
@@ -401,7 +413,7 @@ dev.print.images <- function(basename, dirname,
     filename.pdf <-
       save.image(basename, "pdf", dirname, device.pdf,
                  width = width.pdf, height = height.pdf,
-                 useDingbats = FALSE, as.slide = TRUE, ...,
+                 useDingbats = FALSE, as.slide = pdf.as.slide, ...,
                  clobber = clobber)
   }
 
@@ -416,8 +428,9 @@ dev.print.images <- function(basename, dirname,
 }
 
 save.images <- function(dirpath, basename, image, ..., clobber = FALSE) {
-  cat("Saving images...", file=stderr())
-  plot(image)
+  tryCatch(plot(image),
+           error = function(e) {
+             stop(paste("Error plotting images. Error message:", e$message))
+           })
   dev.print.images(basename, dirpath, ..., clobber = clobber)
-  cat("... done saving images.", file=stderr())
 }
