@@ -1,30 +1,28 @@
 from __future__ import division, with_statement
 
-__version__ = "$Revision: 201 $"
+__version__ = "$Revision: 230 $"
 
 """
-Assorted utility functions and classes common or useful to most of segtools.
+Assorted utility functions and classes common or useful to most of Segtools.
 
 
 Author: Orion Buske <stasis@uw.edu>
 """
 
 import os
-import re
 import sys
 
-from collections import defaultdict
 from contextlib import closing, contextmanager
 from functools import partial
-from genomedata import Genome
 from gzip import open as _gzip_open
-from numpy import array, concatenate, empty, iinfo, logical_and, uint16, uint32, int64
-from operator import itemgetter
-from pkg_resources import resource_filename, resource_string
-from string import Template
+from warnings import warn
+
+from numpy import array, empty, iinfo, ndarray, searchsorted
+from pkg_resources import resource_filename
 from tabdelim import DictReader, DictWriter, ListReader, ListWriter
-from rpy2.robjects import r, rinterface, numpy2ri
-# numpy2ri imported for side-effect
+from rpy2.robjects import r, rinterface
+
+from . import log
 
 try:
     PKG = __package__  # Python 2.6
@@ -34,8 +32,6 @@ except NameError:
 PKG_R = os.extsep.join([PKG, "R"])
 PKG_RESOURCE = os.extsep.join([PKG, "resources"])
 
-from .gff import read_native as read_gff
-
 EXT_GZ = "gz"
 EXT_PDF = "pdf"
 EXT_PNG = "png"
@@ -58,46 +54,8 @@ LABEL_ALL = "all"
 
 THUMB_SIZE = 100
 
-DTYPE_SEGMENT_START = int64
-DTYPE_SEGMENT_END = int64
-DTYPE_SEGMENT_KEY = uint32
-DTYPE_SOURCE_KEY = uint16
-DTYPE_STRAND = '|S1'
-MAXLEN_ID_DTYPE = 20
-DTYPE_GENE_ID = '|S%d' % MAXLEN_ID_DTYPE
-DTYPE_TRANSCRIPT_ID = '|S%d' % MAXLEN_ID_DTYPE
-
-#FEATURE_FIELDS = ["chrom", "source", "name", "start", "end", "score", "strand"]
-
 r_filename = partial(resource_filename, PKG_R)
-template_string = partial(resource_string, PKG_RESOURCE)
-
-
-## Wrapper for gff/gtf feature data
-class Features(object):
-    """
-    sequences: a dict
-      key: chromosome/scaffold name
-      val: segments: numpy.ndarray
-           each row is a struct of: start, end, key, source_key
-           sorted by start, end
-    features: a dict
-      key: int ("feature_key")  (a unique id)
-      val: printable (str or int)  (what's in the actual GFF/GTF file)
-    sources: a dict
-      key: int ("source_key")  (a unique id)
-      val: printable (str or int)  (what's in the actual GFF/GTF file)
-
-    tracks: a list of track names that were used to obtain the segmentation
-    segtool: the tool that was used to obtain this segmentation (e.g. segway)
-    """
-    def __init__(self, sequences, features, sources):
-        self.sequences = sequences
-        self.features = features
-        self.sources = sources
-        # Allow Segmentation-like access
-        self.chromosomes = self.sequences
-        self.labels = self.features
+RError = rinterface.RRuntimeError
 
 class OutputMasker(object):
     """Class to mask the output of a stream.
@@ -174,11 +132,118 @@ class ProgressBar(object):
         self._out.flush()
 
 
+class FeatureScanner(object):
+    def __init__(self, features):
+        self._current = None
+        self._nearest = None
+        if isinstance(features, ndarray):
+            self._iter = iter(features)
+            try:
+                self._current = self._iter.next()
+            except StopIteration:
+                pass
+            else:
+                self._nearest = self._current
+        else:
+            warn("Feature list not of understood type. Treated as empty")
+
+        self._has_next = (self._current is not None)
+        self._overlapping = []
+
+        self._prev_best = None
+        self._next_best = None
+        self._prev_start = None
+
+    @staticmethod
+    def distance(feature1, feature2):
+        """Return the distance between two features (zero if overlap)"""
+        if feature1['end'] <= feature2['start']:
+            return feature2['start'] - feature1['end'] + 1
+        elif feature2['end'] <= feature1['start']:
+            return feature1['start'] - feature2['end'] + 1
+        else:  # Features overlap
+            return 0
+
+    @staticmethod
+    def are_overlapping(feature1, feature2):
+        """Return the whether or not two features overlap"""
+        return feature1['start'] < feature2['end'] and \
+                feature2['start'] < feature1['end']
+
+    def _seek(self, segment):
+        # Seek internal records up to segment, updated self._overlapping
+        # and self._nearest
+
+        # Clear out any non-overlapping features from self._overlapping
+        if self._overlapping:
+            i = 0
+            while i < len(self._overlapping):
+                if self.are_overlapping(segment, self._overlapping[i]):
+                    i += 1
+                else:
+                    del self._overlapping[i]
+
+        # If any are overlapping, nearest is 0 distance
+        if self._overlapping:
+            nearest_dist = 0
+            self._nearest = self._overlapping[0]
+        elif self._nearest is not None:
+            # Closest feature will start as the closest from the last iteration
+            nearest_dist = self.distance(self._nearest, segment)
+        else:
+            assert not self._has_next
+
+        # Scan through features until we get past segment
+        if self._has_next:
+            segment_start = segment['start']
+            segment_end = segment['end']
+            while self._current['start'] < segment_end:
+                # dist will be negative if this overlaps
+                dist = segment_start - self._current['end'] + 1
+                if dist < 0:
+                    self._overlapping.append(self._current)
+
+                if dist < nearest_dist:
+                    self._nearest = self._current
+                    nearest_dist = dist
+
+                try:
+                    self._current = self._iter.next()
+                except StopIteration:
+                    self._has_next = False
+                    break
+
+            # Check distance of first feature past segment as well
+            dist = self._current['start'] - segment_end + 1
+            if dist < nearest_dist:
+                self._nearest = self._current
+                nearest_dist = dist
+
+    def nearest(self, segment):
+        """Return a list of the nearest features for the next segment
+
+        This will contain a single feature if no features overlap the given
+        segment, else it will be a list of features that overlap.
+
+        If no features were found at all, this will be None.
+
+        Should be called with in-order segments
+
+        """
+        self._seek(segment)
+        if self._overlapping:
+            return self._overlapping
+        elif self._nearest is None:
+            return None
+        else:
+            return self._nearest
+
+
 ## UTILITY FUNCTIONS
 
 ## Die with error message
 def die(msg="Unexpected error"):
-    print >> sys.stderr, "\nERROR: %s\n" % msg
+    log("\nERROR: %s\n" % msg)
     sys.exit(1)
 
 def inverse_dict(d):
@@ -230,26 +295,50 @@ def fill_array(scalar, shape, dtype=None, *args, **kwargs):
     res.fill(scalar)
     return res
 
-
-## XXX: No known usage
-@contextmanager
-def none_contextmanager():
-    yield None
-
 def r_source(filename):
-    """
-    Simplify importing R source in the package
-    """
+    """Simplify importing R source in the package"""
+
     try:
         r.source(r_filename(filename))
-    except rinterface.RRuntimeError:
+    except RError:
         die("Failed to load R package: %s\n" % filename)
 
-def template_substitute(filename):
+def r_call(func, *args, **kwargs):
+    """Safer way to call R functions (without importing all that junk)
+
+    None values are substituted with ""
     """
-    Simplify import resource strings in the package
+    from rpy2.robjects import numpy2ri
+    # numpy2ri imported for side-effect
+
+    # Make sure there are no None values in args or kwargs
+    # rpy2 currently doesn't support passing Nones, so replace them with ""
+    for arg_i, arg in enumerate(args):
+        if arg is None:
+            args[arg_i] = ""
+
+    for key, val in kwargs.iteritems():
+        if val is None:
+            kwargs[key] = ""
+
+    return r[func](*args, **kwargs)
+
+def r_plot(func, *args, **kwargs):
+    """Safely call R plotting function (with verbose print option)
+
+    Calls r_call
+    Argument "verbose" is caught by r_plot and not passed on to R functions
     """
-    return Template(template_string(filename)).substitute
+
+    # Grab verbose value, with default to True
+    verbose = kwargs.pop("verbose", True)
+
+    log("Saving plots with R function: %r..." % func, verbose, end="")
+
+    result = r_call(func, *args, **kwargs)
+    log(" done", verbose)
+
+    return result
 
 @contextmanager
 def tab_saver(dirpath, basename, fieldnames=None, header=True,
@@ -263,8 +352,7 @@ def tab_saver(dirpath, basename, fieldnames=None, header=True,
       Comment line will start with '# ' and then will be a space-delimited
       list of <field>=<value> pairs, one for each element of the dict.
     """
-    if verbose:
-        print >>sys.stderr, "Saving tab file...",
+    log("Saving tab file...", verbose, end="")
 
     outfilename = make_tabfilename(dirpath, basename)
     check_clobber(outfilename, clobber)
@@ -280,8 +368,7 @@ def tab_saver(dirpath, basename, fieldnames=None, header=True,
         else:
             yield ListWriter(outfile)
 
-    if verbose:
-        print >>sys.stderr, "done"
+    log(" done", verbose)
 
 @contextmanager
 def tab_reader(dirpath, basename, verbose=True, fieldnames=False):
@@ -293,8 +380,7 @@ def tab_reader(dirpath, basename, verbose=True, fieldnames=False):
 
     metadata: a dict describing the data included in the comment line.
     """
-    if verbose:
-        print >>sys.stderr, "Reading tab file...",
+    log("Reading tab file...", verbose, end="")
 
     infilename = make_tabfilename(dirpath, basename)
     if not os.path.isfile(infilename):
@@ -317,71 +403,14 @@ def tab_reader(dirpath, basename, verbose=True, fieldnames=False):
         else:
             yield ListReader(infile), metadata
 
-    if verbose:
-        print >>sys.stderr, "done"
-
-@contextmanager
-def image_saver(dirpath, basename, clobber=False, verbose=True,
-                downsample=False, **kwargs):
-    """
-    Generator to save an R plot to both png and pdf with only one plot
-    Yields to caller to plot, then saves images
-    """
-    if verbose:
-        print >>sys.stderr, "Creating images...",
-
-    png_filename = make_pngfilename(dirpath, basename)
-    check_clobber(png_filename, clobber)
-    try:
-        r.png(png_filename, **kwargs)
-        png_device = r["dev.cur"]()
-        r["dev.control"]("enable")
-    except rinterface.RRuntimeError:
-        die('Image creation failed.\nIf unable to start PNG device, try'
-            ' setting (export/setenv) variable DISPLAY to ":1" from the'
-            ' (bash/c) shell before re-running validation.')
-
-    yield  # Wait for plot
-
-    # Use R routine to create all the other images (pdf, slide, etc)
-    try:
-        r["dev.print.images"](basename=basename, dirname=dirpath,
-                              downsample=downsample, clobber=clobber, **kwargs)
-        r["dev.off"](png_device)
-    except rinterface.RRuntimeError:
-        print >> sys.stderr, 'ERROR: Images might not have been saved!'
-
-    if verbose:
-        print >>sys.stderr, "done"
-
-
-## Maps label_keys over segments
-def map_segments(segments, labels, chrom_size):
-    """
-    converts a segment mapping into label_keys at every nucleotide position
-        in the chromosome
-
-    e.g.  segment labeled 0 at position [0,4) followed by
-          segment labeled 1 at position [4,7) gets converted into:
-          0000111
-    """
-    segments_dtype = segments['key'].dtype  # MA: the data type of segments
-    segments_dtype_max = iinfo(segments_dtype).max  # sentinel value
-    # MA: the maximum value supported by the type
-    assert segments_dtype_max not in labels.keys()
-
-    res = fill_array(segments_dtype_max, (chrom_size,), segments_dtype)
-
-    # will overwrite in overlapping case
-    for start, end, key in segments:
-        res[start:end] = key
-
-    return res
+    log(" done", verbose)
 
 def map_segment_label(segments, range):
     """Flatten segments to a segment label per base vector
 
-    Returns a numpy.array of the segment key at every base
+    Returns a tuple of:
+      numpy.array of the segment key at every base
+      sentinal value assigned to non-overlapped bases
 
     The given range must either be a tuple of (start, end), or
       have 'start' and 'end' attributes, and the returned array
@@ -410,13 +439,16 @@ def map_segment_label(segments, range):
         return res
 
     # will overwrite in overlapping case
-    for start, end, key in segments:
+    for segment in segments:
+        start = segment['start']
+        end = segment['end']
+        key = segment['key']
         if start < map_end and end > map_start:
             start_i = max(start - map_start, 0)
             end_i = min(end - map_start, map_size)
             res[start_i:end_i] = key
 
-    return res
+    return res, sentinal
 
 ## Yields segment and the continuous corresponding to it, for each segment
 ##   in the chromosome inside of a supercontig
@@ -443,7 +475,7 @@ def iter_segments_continuous(chromosome, segmentation, verbose=True):
 
         while supercontig is None or start >= supercontig.end:
             try:
-                # Raise StopIteration if out of supercontigs
+                # Raises StopIteration if out of supercontigs
                 supercontig, continuous = supercontig_iter.next()
             except StopIteration:
                 if verbose:
@@ -480,14 +512,15 @@ def iter_supercontig_segments(chromosome, segmentation, verbose=True):
         raise StopIteration
 
     for supercontig in chromosome:
-        if verbose:
-            print >>sys.stderr, "\t\t%s" % supercontig
-
-        rows = logical_and(segments['start'] < supercontig.end,
-                           segments['end'] > supercontig.start)
-        cur_segments = segments[rows]
-        if cur_segments.shape[0] > 0:
-            yield supercontig, segments[rows]
+        # Since it's a segmentation, there are no overlapping segments.
+        # Thus, sorting by the start also sorts by the end, and we can
+        # binary search on both to get the bounds of the overlapping segments.
+        # Overlapping segments are guaranteed to be a contiguous set of rows
+        # in the segment array for this reason.
+        start = searchsorted(segments['end'], supercontig.start, side="right")
+        end = searchsorted(segments['start'], supercontig.end, side="left")
+        if end > start:
+            yield supercontig, segments[start:end]
 
 ## Ensures output directory exists and has appropriate permissions
 def setup_directory(dirname):
@@ -539,7 +572,7 @@ def get_ordered_labels(labels, mnemonics=[]):
 ## every old_label, and their ordering specifies the desired display order
 ## Labels should be a dict of label_keys -> label strings
 def map_mnemonics(labels, mnemonicfilename, field="new"):
-    if mnemonicfilename is None:
+    if not mnemonicfilename:
         return []
 
     label_order, label_data = load_mnemonics(mnemonicfilename)
@@ -601,190 +634,3 @@ def load_mnemonics(filename):
             label_data[label_index] = row
 
     return (label_order, label_data)
-
-## Parses gff file for features
-def load_gff_data(gff_filename, sort=True):
-    '''
-    Expects data in GFF format (1-indexed locations):
-    CHROM<tab>source<tab>feature<tab>START<tab>END<tab>score<tab>STRAND
-    chrom, start, end, and strand are required
-    strand can be '+'/'-' or '.', but not both in the same file
-
-    File may be gzip'd, but if so, must have .gz as the extension
-
-    Returns gffdata
-    gffdata: a dict
-      key: string chromosome name (e.g. "chr13")
-      val: a list of dicts (ordered by the start index, ascending)
-        key: "start", "end", "strand", "source"
-        val: the associated data item, if it exists
-             string for strand
-             int (zero-indexed) for start and end (exclusive)
-    '''
-    data = defaultdict(list)
-    stranded = None
-    with maybe_gzip_open(gff_filename) as infile:
-        for line in infile:
-            # Ignore comments
-            if line.startswith("#"): continue
-
-            # Parse tokens from GFF line
-            try:
-                fields = {}
-                tokens = line.strip().split("\t")
-
-                chrom = tokens[0]
-                fields["source"] = tokens[1]
-                fields["name"] = tokens[2]
-                fields["start"] = int(tokens[3]) - 1  # Make zero-indexed
-                # Make zero-indexed and exclusive (these cancel out)
-                fields["end"] = int(tokens[4])
-
-                try:
-                    strand = tokens[6]
-                except IndexError:
-                    strand = "."
-
-                if strand == "+" or strand == "-":
-                    assert stranded or stranded is None
-                    stranded = True
-                else:
-                    assert not stranded  # Don't have both +/- and .
-                    strand = "."  # N/A
-                    stranded = False
-
-                fields["strand"] = strand
-
-                data[chrom].append(fields)
-            except (IndexError, ValueError):
-                die("Error parsing fields from feature line:\n\t%s" % line)
-
-    if sort:
-        # Sort features by ascending start
-        for chrom_features in data.itervalues():
-            chrom_features.sort(key=itemgetter("start"))
-
-    return data
-
-def gff2arraydict(filename, gtf=False, sort=None, verbose=True):
-    """Parses a gff/gtf file and returns a dict of numpy arrays for each chrom.
-
-    Default behavior is to sort features unless 'gtf' is True
-
-    Returns labels, sequences, where:
-      labels: a dict from feature_key -> feature string
-      sequences: a dict from seqname -> features (numpy array)
-        (features: structured array with start, end, key, and source fields.
-         If stranded, a strand field is included.
-         If gtf-mode, gene_id and transcript_id fields are included.)
-    """
-    if sort is None:
-        sort = not gtf
-
-    data = defaultdict(list)  # A dictionary-like object
-    feature_dict = {}
-    source_dict = {}
-    last_segment_start = {}  # per seq
-    unsorted_seqs = set()
-    stranded = None
-    with maybe_gzip_open(filename) as infile:
-        for datum in read_gff(infile, gtf=gtf):
-            try:
-                if datum.start < last_segment_start[datum.seqname]:
-                    unsorted_seqs.add(datum.seqname)
-            except KeyError:
-                pass
-
-            feature = str(datum.feature)
-            try:  # Lookup feature key
-                feature_key = feature_dict[feature]
-            except KeyError:  # Map new feature to key
-                feature_key = len(feature_dict)
-                feature_dict[feature] = feature_key
-
-            source = str(datum.source)
-            try:  # Lookup source key
-                source_key = source_dict[source]
-            except KeyError:  # Map new source to key
-                source_key = len(source_dict)
-                source_dict[source] = source_key
-
-            try:
-                strand = datum.strand
-            except AttributeError:
-                strand = "."
-
-            if strand == "+" or strand == "-":
-                assert stranded is None or stranded
-                stranded = True
-            else:
-                assert not stranded
-                stranded = False
-
-            feature = [datum.start, datum.end, feature_key, source_key]
-            if stranded:
-                feature.append(datum.strand)
-
-            if gtf:
-                gene_id = datum.attributes["gene_id"]
-                transcript_id = datum.attributes["transcript_id"]
-                if len(gene_id) > MAXLEN_ID_DTYPE:
-                    raise ValueError("gene_id field was too long (over \
-%d characters): %s" % (MAXLEN_ID_DTYPE, gene_id))
-                elif len(transcript_id) > MAXLEN_ID_DTYPE:
-                    raise ValueError("transcript_id field was too long (over \
-%d characters): %s" % (MAXLEN_ID_DTYPE, transcript_id))
-                feature.extend([gene_id, transcript_id])
-
-            data[datum.seqname].append(tuple(feature))
-            last_segment_start[datum.seqname] = datum.start
-
-    # Create reverse dict for return
-    features = inverse_dict(feature_dict)
-    sources = inverse_dict(source_dict)
-
-    # convert lists of tuples to array
-    dtype = [('start', DTYPE_SEGMENT_START),
-             ('end', DTYPE_SEGMENT_END),
-             ('key', DTYPE_SEGMENT_KEY),
-             ('source_key', DTYPE_SOURCE_KEY)]
-    if stranded:
-        dtype.append(('strand', DTYPE_STRAND))
-    if gtf:
-        dtype.extend([('gene_id', DTYPE_GENE_ID),
-                      ('transcript_id', DTYPE_TRANSCRIPT_ID)])
-
-    sequences = dict((seq, array(segments, dtype=dtype))
-                       for seq, segments in data.iteritems())
-
-    # Sort segments within each chromosome
-    for seq, segments in sequences.iteritems():
-        if sort and seq in unsorted_seqs:
-            if verbose:
-                print >>sys.stderr, \
-                    "Segments were unsorted relative to %s. Sorting..." % seq,
-            segments.sort(order=['start'])
-            if verbose:
-                print >>sys.stderr, "done"
-
-    return features, sources, sequences
-
-def load_features(filename, verbose=True, *args, **kwargs):
-    if verbose:
-        print >>sys.stderr, "Loading features from: %s..." % filename
-
-    features, sources, sequences = \
-        gff2arraydict(filename, verbose=verbose, *args, **kwargs)
-
-    if verbose:
-        print >>sys.stderr, "\tFound %d sources:" % len(sources)
-        for source_key, source in sources.iteritems():
-            print >>sys.stderr, '\t\t"%s" -> %d' % (source, source_key)
-
-        print >>sys.stderr, "\tFound %d feature classes" % len(features)
-        for key, feature in features.iteritems():
-            print >>sys.stderr, '\t\t"%s" -> %d' % (feature, key)
-
-        print >>sys.stderr, "done"
-
-    return Features(sequences, features, sources)
