@@ -2,8 +2,6 @@
 from __future__ import division, with_statement
 
 """
-validate.py:
-
 Provides command-line and package entry points for analyzing the segment
 length distribution in a provided BED-formatted segmentation.
 
@@ -13,17 +11,16 @@ length distribution in a provided BED-formatted segmentation.
 # Must be the same as the folder containing this script
 MODULE="length_distribution"
 
-__version__ = "$Revision: 192 $"
+__version__ = "$Revision: 250 $"
 
 import os
 import sys
 
 from collections import defaultdict
 from numpy import concatenate, median
-from rpy2.robjects import r, numpy2ri
 
 from . import Segmentation
-from .common import die, get_ordered_labels, image_saver, LABEL_ALL, make_tabfilename, map_mnemonics, r_source, setup_directory, tab_saver
+from .common import die, get_ordered_labels, LABEL_ALL, make_tabfilename, r_plot, r_source, setup_directory, tab_saver
 
 from .html import save_html_div
 
@@ -84,12 +81,11 @@ def make_size_row(label, lengths, num_bp, total_bp):
             "frac.bp": "%.3f" % (num_bp / total_bp)}
 
 ## Saves the length summary data to a tab file, using mnemonics if specified
-def save_size_tab(lengths, labels, num_bp, dirpath,
-                     namebase=NAMEBASE_SIZES, clobber=False, mnemonics=None):
-    # Get mnemonic ordering and labels, if specified
-    ordered_keys, labels = get_ordered_labels(labels, mnemonics)
+def save_size_tab(lengths, labels, num_bp, dirpath, verbose=True,
+                  namebase=NAMEBASE_SIZES, clobber=False):
+    ordered_keys, labels = get_ordered_labels(labels)
     with tab_saver(dirpath, namebase, FIELDNAMES_SUMMARY,
-                   clobber=clobber) as saver:
+                   clobber=clobber, verbose=verbose) as saver:
         # "all" row first
         total_bp = sum(num_bp.itervalues())
         all_lengths = concatenate(lengths.values())
@@ -110,7 +106,7 @@ def make_row(label, length):
     return {"label": label,
             "length": length}
 
-def save_tab(lengths, labels, num_bp, dirpath, clobber=False):
+def save_tab(lengths, labels, num_bp, dirpath, clobber=False, verbose=True):
     # fix order for iterating through dict; must be consistent
     label_keys = sorted(labels.keys())
 
@@ -123,14 +119,15 @@ def save_tab(lengths, labels, num_bp, dirpath, clobber=False):
     labels_array = concatenate([[labels[label_key]] * len(lengths[label_key])
                                for label_key in label_keys])
 
-    with tab_saver(dirpath, NAMEBASE, FIELDNAMES, clobber=clobber) as saver:
+    with tab_saver(dirpath, NAMEBASE, FIELDNAMES, verbose=verbose,
+                   clobber=clobber) as saver:
         for label, length in zip(labels_array, lengths_array):
             row = make_row(label, length)
             saver.writerow(row)
 
 ## Generates and saves an R plot of the length distributions
-def save_plot(dirpath, namebase=NAMEBASE, clobber=False,
-              mnemonic_file=""):
+def save_plot(dirpath, namebase=NAMEBASE, clobber=False, verbose=True,
+              mnemonic_file=None):
     start_R()
 
     # Load data from corresponding tab file
@@ -138,15 +135,12 @@ def save_plot(dirpath, namebase=NAMEBASE, clobber=False,
     if not os.path.isfile(tabfilename):
         die("Unable to find tab file: %s" % tabfilename)
 
-    if not mnemonic_file:
-        mnemonic_file = ""  # None cannot be passed to R
-
-    r["save.length"](dirpath, namebase, tabfilename,
-                     mnemonic_file=mnemonic_file,
-                     clobber=clobber)
+    r_plot("save.length", dirpath, namebase, tabfilename,
+           mnemonic_file=mnemonic_file, clobber=clobber, verbose=verbose)
 
 ## Generates and saves an R plot of the length distributions
-def save_size_plot(dirpath, namebase=NAMEBASE_SIZES, clobber=False):
+def save_size_plot(dirpath, namebase=NAMEBASE_SIZES, clobber=False,
+                   verbose=True, mnemonic_file=None):
     start_R()
 
     # Load data from corresponding tab file
@@ -154,72 +148,78 @@ def save_size_plot(dirpath, namebase=NAMEBASE_SIZES, clobber=False):
     if not os.path.isfile(tabfilename):
         die("Unable to find tab file: %s" % tabfilename)
 
-    r["save.segment.sizes"](dirpath, namebase, tabfilename,
-                            clobber=clobber)
+    r_plot("save.segment.sizes", dirpath, namebase, tabfilename,
+           mnemonic_file=mnemonic_file, clobber=clobber, verbose=verbose)
 
-def save_html(dirpath, clobber=False):
+def save_html(dirpath, clobber=False, mnemonicfile=None):
     extra_namebases = {"sizes": NAMEBASE_SIZES}
     save_html_div(TEMPLATE_FILENAME, dirpath, namebase=NAMEBASE,
-                  extra_namebases=extra_namebases,
-                  tablenamebase=NAMEBASE_SIZES, module=MODULE,
+                  extra_namebases=extra_namebases, mnemonicfile=mnemonicfile,
+                  tables={"":NAMEBASE_SIZES}, module=MODULE,
                   clobber=clobber, title=HTML_TITLE)
 
 ## Package entry point
 def validate(bedfilename, dirpath, clobber=False, replot=False, noplot=False,
-             mnemonicfilename=None):
+             verbose=True, mnemonic_file=None):
     if not replot:
         setup_directory(dirpath)
-        segmentation = Segmentation(bedfilename)
+        segmentation = Segmentation(bedfilename, verbose=verbose)
 
         labels = segmentation.labels
-        mnemonics = map_mnemonics(labels, mnemonicfilename)
 
         lengths, num_bp=segmentation_lengths(segmentation)
-        save_tab(lengths, labels, num_bp, dirpath, clobber=clobber)
+        save_tab(lengths, labels, num_bp, dirpath,
+                 clobber=clobber, verbose=verbose)
         save_size_tab(lengths, labels, num_bp, dirpath,
-                         clobber=clobber, mnemonics=mnemonics)
+                      clobber=clobber, verbose=verbose)
 
     if not noplot:
-        save_plot(dirpath, clobber=clobber, mnemonic_file=mnemonicfilename)
-        save_size_plot(dirpath, clobber=clobber)
+        save_plot(dirpath, mnemonic_file=mnemonic_file,
+                  clobber=clobber, verbose=verbose)
+        save_size_plot(dirpath, clobber=clobber, verbose=verbose,
+                       mnemonic_file=mnemonic_file)
 
-    save_html(dirpath, clobber=clobber)
+    save_html(dirpath, clobber=clobber, mnemonicfile=mnemonic_file)
 
 def parse_options(args):
-    from optparse import OptionParser
+    from optparse import OptionGroup, OptionParser
 
-    usage = "%prog [OPTIONS] BEDFILE"
+    usage = "%prog [OPTIONS] SEGMENTATION"
     version = "%%prog %s" % __version__
     parser = OptionParser(usage=usage, version=version)
 
-    parser.add_option("--clobber", action="store_true",
-                      dest="clobber", default=False,
-                      help="Overwrite existing output files if the specified"
-                      " directory already exists.")
-    parser.add_option("--replot", action="store_true",
-                      dest="replot", default=False,
-                      help="Load data from output tab files and"
-                      " regenerate plots instead of recomputing data")
-    parser.add_option("--noplot", action="store_true",
-                      dest="noplot", default=False,
-                      help="Do not generate plots")
+    group = OptionGroup(parser, "Flags")
+    group.add_option("--clobber", action="store_true",
+                     dest="clobber", default=False,
+                     help="Overwrite existing output files if the specified"
+                     " directory already exists.")
+    group.add_option("-q", "--quiet", action="store_false",
+                     dest="verbose", default=True,
+                     help="Do not print diagnostic messages.")
+    group.add_option("--replot", action="store_true",
+                     dest="replot", default=False,
+                     help="Load data from output tab files and"
+                     " regenerate plots instead of recomputing data")
+    group.add_option("--noplot", action="store_true",
+                     dest="noplot", default=False,
+                     help="Do not generate plots")
+    parser.add_option_group(group)
 
-    parser.add_option("--mnemonic-file", dest="mnemonicfilename",
-                      default=None,
-                      help="If specified, labels will be shown using"
-                      " mnemonics found in this file")
-    parser.add_option("-o", "--outdir",
-                      dest="outdir", default="%s" % MODULE,
-                      help="File output directory (will be created"
-                      " if it does not exist) [default: %default]")
+    group = OptionGroup(parser, "Output")
+    group.add_option("-m", "--mnemonic-file", dest="mnemonic_file",
+                     default=None, metavar="FILE",
+                     help="If specified, labels will be shown using"
+                     " mnemonics found in FILE.")
+    group.add_option("-o", "--outdir",
+                     dest="outdir", default="%s" % MODULE, metavar="DIR",
+                     help="File output directory (will be created"
+                     " if it does not exist) [default: %default]")
+    parser.add_option_group(group)
 
     (options, args) = parser.parse_args(args)
 
-    if len(args) < 1:
-        parser.error("Insufficient number of arguments")
-
-    if options.noplot and options.replot:
-        parser.error("noplot and replot are contradictory")
+    if len(args) != 1:
+        parser.error("Inappropriate number of arguments")
 
     return (options, args)
 
@@ -228,9 +228,12 @@ def main(args=sys.argv[1:]):
     (options, args) = parse_options(args)
     bedfilename = args[0]
 
-    validate(bedfilename, options.outdir, clobber=options.clobber,
-             replot=options.replot, noplot=options.noplot,
-             mnemonicfilename=options.mnemonicfilename)
+    kwargs = {"clobber": options.clobber,
+              "verbose": options.verbose,
+              "replot": options.replot,
+              "noplot": options.noplot,
+              "mnemonic_file": options.mnemonic_file}
+    validate(bedfilename, options.outdir, **kwargs)
 
 if __name__ == "__main__":
     sys.exit(main())
