@@ -1,7 +1,7 @@
 #!/bin/env python
 from __future__ import division, with_statement
 
-__version__ = "$Revision: 190 $"
+__version__ = "$Revision: 228 $"
 
 """
 mnemonics.py
@@ -11,12 +11,11 @@ mnemonics.py
 import os
 import sys
 
-from rpy2.robjects import r, rinterface, numpy2ri
-
-from .common import check_clobber, r_source
+from . import log
+from .common import check_clobber, r_call, r_source, RError
 
 def create_mnemonic_file(datafile, dirpath, clobber=False,
-                         namebase=None, gmtk=False):
+                         namebase=None, gmtk=False, verbose=True):
     """Generate a mnemonic file with R clustering code.
 
     Datafile can either be a signal stats file or a GMTK params file
@@ -43,15 +42,17 @@ def create_mnemonic_file(datafile, dirpath, clobber=False,
 
     # Create mnemonic file via R
     try:
-        r_filename = r["make.mnemonic.file"](datafile, filename,
-                                             gmtk=gmtk)
+        r_filename = r_call("make.mnemonic.file",
+                            datafile, filename, gmtk=gmtk)
+        # Peel off rpy2 layers
         mnemonicfilename = str(r_filename[0])
 
-        print >>sys.stderr, "Generated mnemonic file: %s" % mnemonicfilename
+        log("Created mnemonic file: %s" % mnemonicfilename, verbose)
+
         return mnemonicfilename
-    except rinterface.RRuntimeError:
-        print >>sys.stderr, ("ERROR: Failed to create mnemonic file."
-                             " Continuing without mnemonics.")
+    except RError:
+        log("ERROR: Failed to create mnemonic file."
+            " Continuing without mnemonics.")
         return None
 
 if __name__ == "__main__":
